#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `nombot` package."""


import unittest

#from trader import main


class TestNombot(unittest.TestCase):
    """Tests for `nombot` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_main(self):
        """Test main trader"""
